use eframe::{egui, epi};
use std::process::Command;
use crate::{Sound, file_manager};

pub struct State {
    device: rodio::Device,
    sounds: Vec<Sound>,
}

impl Default for State {
    fn default() -> Self {
        let device = rodio::default_output_device().unwrap();
        let mut sounds = crate::get_sounds(&device);

        Self { device, sounds }
    }
}

impl epi::App for State {
    fn update(&mut self, ctx: &egui::CtxRef, _frame: &mut epi::Frame<'_>) {
        let State { device: _device, sounds} = self;

        for sound in sounds.into_iter() {
            if sound.volume > 0.0 {
                sound.sink.set_volume(sound.volume as f32);
                sound.sink.play();
            } else {
                sound.sink.pause();
            }
        }

        egui::CentralPanel::default().show(ctx, |ui| {
            for sound in sounds {
                ui.separator();
                ui.label(&sound.name);

                ui.add(
                    egui::Slider::new(
                        &mut sound.volume,
                        0.0..=1.0
                    ).show_value(false)
                );
            }

            ui.separator();

            if ui.button("manage sounds").clicked() {
                let mut file_manager = Command::new(file_manager());
                file_manager.arg(crate::get_data_dir());
                file_manager.output().unwrap();
            }
        });
    }

    fn name(&self) -> &str {
        "lull"
    }
}
